@@ -21,44 +21,38 @@
 #ifndef RESOURCESMODEL_H
 #define RESOURCESMODEL_H
 
-#include <QtCore/QModelIndex>
+#include <QSet>
 #include <QVector>
+#include <QTimer>
 
 #include "discovercommon_export.h"
 #include "AbstractResourcesBackend.h"
 
-class AbstractResource;
-class AbstractResourcesBackend;
+class DISCOVERCOMMON_EXPORT AggregatedResultsStream : public ResultsStream
+{
+Q_OBJECT
+public:
+    AggregatedResultsStream(const QSet<ResultsStream*>& streams);
 
-class DISCOVERCOMMON_EXPORT ResourcesModel : public QAbstractListModel
+Q_SIGNALS:
+    void finished();
+
+private:
+    void addResults(const QVector<AbstractResource*>& res);
+    void emitResults();
+    void destruction(QObject* obj);
+
+    QSet<QObject*> m_streams;
+    QVector<AbstractResource*> m_results;
+    QTimer m_delayedEmission;
+};
+
+class DISCOVERCOMMON_EXPORT ResourcesModel : public QObject
 {
     Q_OBJECT
     Q_PROPERTY(int updatesCount READ updatesCount NOTIFY updatesCountChanged)
     Q_PROPERTY(bool isFetching READ isFetching NOTIFY fetchingChanged)
     public:
-        enum Roles {
-            NameRole = Qt::UserRole,
-            IconRole,
-            CommentRole,
-            StateRole,
-            RatingRole,
-            RatingPointsRole,
-            RatingCountRole,
-            SortableRatingRole,
-            ActiveRole,
-            InstalledRole,
-            ApplicationRole,
-            OriginRole,
-            CanUpgrade,
-            PackageNameRole,
-            IsTechnicalRole,
-            CategoryRole,
-            CategoryDisplayRole,
-            SectionRole,
-            MimeTypes,
-            SizeRole,
-            LongDescriptionRole
-        };
         /** This constructor should be only used by unit tests.
          *  @p backendName defines what backend will be loaded when the backend is constructed.
          */
@@ -66,27 +60,21 @@ class DISCOVERCOMMON_EXPORT ResourcesModel : public QAbstractListModel
         static ResourcesModel* global();
         ~ResourcesModel() override;
         
-        QVariant data(const QModelIndex& index, int role = Qt::DisplayRole) const override;
-        int rowCount(const QModelIndex& parent = QModelIndex()) const override;
-        
-        AbstractResource* resourceAt(int row) const;
-        QModelIndex resourceIndex(AbstractResource* res) const;
         QVector< AbstractResourcesBackend* > backends() const;
         int updatesCount() const;
-        QMap< int, QVariant > itemData(const QModelIndex& index) const override;
         
-        Q_SCRIPTABLE AbstractResource* resourceByPackageName(const QString& name);
-
         void integrateActions(KActionCollection* w);
         
         bool isBusy() const;
         bool isFetching() const;
         QList<QAction*> messageActions() const;
         
-        QHash<int, QByteArray> roleNames() const override;
-
         Q_SCRIPTABLE bool isExtended(const QString &id);
 
+        AggregatedResultsStream* findResourceByPackageName(const QUrl& search);
+        AggregatedResultsStream* search(const AbstractResourcesBackend::Filters &search);
+        AbstractResource* resourceForFile(const QUrl &/*url*/);
+
     public Q_SLOTS:
         void installApplication(AbstractResource* app, const AddonList& addons);
         void installApplication(AbstractResource* app);
@@ -97,21 +85,18 @@ class DISCOVERCOMMON_EXPORT ResourcesModel : public QAbstractListModel
         void allInitialized();
         void backendsChanged();
         void updatesCountChanged();
-        void searchInvalidated();
-        void resourceDataChanged(AbstractResource* res, const QVector<QByteArray> &properties);
+        void backendDataChanged(AbstractResourcesBackend* backend, const QVector<QByteArray>& properties);
+        void resourceDataChanged(AbstractResource* resource, const QVector<QByteArray>& properties);
+        void resourceRemoved(AbstractResource* resource);
+        void passiveMessage(const QString &message);
 
     private Q_SLOTS:
-        void resetBackend(AbstractResourcesBackend* backend);
-        void cleanBackend(AbstractResourcesBackend* backend);
         void callerFetchingChanged();
         void updateCaller(const QVector<QByteArray>& properties);
         void registerAllBackends();
-        void resourceChangedByTransaction(Transaction* t);
-        void emitResourceChanges(AbstractResource* res, const QVector<QByteArray> &properties);
 
     private:
         QVector<int> propertiesToRoles(const QVector<QByteArray>& propertyNames) const;
-        int rowsBeforeBackend(AbstractResourcesBackend* backend, QVector<QVector<AbstractResource*>>::iterator& backendsResources);
 
         ///@p initialize tells if all backends load will be triggered on construction
         explicit ResourcesModel(QObject* parent=nullptr, bool load = true);
@@ -120,10 +105,8 @@ class DISCOVERCOMMON_EXPORT ResourcesModel : public QAbstractListModel
         void registerBackendByName(const QString& name);
 
         QVector< AbstractResourcesBackend* > m_backends;
-        QVector< QVector<AbstractResource*> > m_resources;
         int m_initializingBackends;
         KActionCollection* m_actionCollection;
-        const QHash<int, QByteArray> m_roles;
 
         static ResourcesModel* s_self;
 };
